/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.3     10:08:58 10/29/2012 Gary Yu                                *
* File Created                                                                *
* FPGA_ID_SEL,//r/w                                                           *       
* FPGA_CTRL_SEL,//w                                                           *
* FPGA_STATUS_SEL,//r                                                         *
* Revision 1.4     10:08:58 12/11/2012 Gary Yu                                *
* DAC_CTRL_SEL,//w                                                            *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/
module add_dec
(
actpls, 
adflag,
addr, 
rst,

FPGA_ID_SEL,      
FPGA_CTRL_SEL,
FPGA_STATUS_SEL,

DAC_CTRL_SEL,
CAP_RST_SEL,

I2C_ACSS_SEL
);

parameter my_register_number          = 6;
    
input       actpls; 
input       adflag;
input       rst;

input [7:0] addr; 

output FPGA_ID_SEL;       
output FPGA_CTRL_SEL;
output FPGA_STATUS_SEL;

output DAC_CTRL_SEL;
output CAP_RST_SEL;

output I2C_ACSS_SEL;

reg FPGA_ID_SEL;     
reg FPGA_CTRL_SEL;
reg FPGA_STATUS_SEL;

reg DAC_CTRL_SEL;
reg CAP_RST_SEL;

reg I2C_ACSS_SEL;

  // ----------------------------------------------------
  // Address Decoder
  // ----------------------------------------------------
 
always @ (posedge actpls or negedge rst)   
  if (~rst)
  begin
  
    FPGA_ID_SEL     <= #1 1'b0;   
    FPGA_CTRL_SEL   <= #1 1'b0;  
    FPGA_STATUS_SEL <= #1 1'b0;
    DAC_CTRL_SEL    <= #1 1'b0;
    CAP_RST_SEL     <= #1 1'b0;
    I2C_ACSS_SEL    <= #1 1'b0;    
  end
  else if (adflag)
  begin
    casex ({addr})
    8'h00: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 0;
    8'h01: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 1;
    8'h02: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 2;
           
    8'h03: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 3;
    
    8'h04: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 4;
         
    8'h11: {
             FPGA_ID_SEL,     
             FPGA_CTRL_SEL,
             FPGA_STATUS_SEL,
             DAC_CTRL_SEL,
             CAP_RST_SEL,
             I2C_ACSS_SEL
           } <= #1 {1'b1, {(my_register_number - 1){1'b0}}} >> 5;               
    endcase 
  end  

endmodule
