/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module altbram_ctrl #
(
parameter   BRAM_ADDR_WIDTH = 17
)
(
//playback control
playback_data,
playback_done,
bclk,
bram_b_clk_ctrl,
tx1_ctrl,

//loading control
rxfn,
loading_done,
rdn,
aclk,
bram_wr_window,
bram_a_addr,//gyu fifo handshake
bram_wr_stop,

arst,
brst
  );
//playback control
output playback_data;
input playback_done;
input bclk;
output bram_b_clk_ctrl;
output tx1_ctrl;

//loading control
input rxfn;
input loading_done;
output rdn;
input aclk;
output bram_wr_window;
input [BRAM_ADDR_WIDTH - 1:0] bram_a_addr;//gyu fifo handshake
output bram_wr_stop;

input arst;
input brst;

//playback control
reg early_playback_data;
reg pre_playback_data;
reg playback_data;    
reg [4:0] playback_done_reg;
always @ (posedge bclk or negedge brst)
begin
  if(~brst)
    playback_done_reg <= #1 5'b0;
  else if (playback_data)
  begin
    playback_done_reg[0] <= #1 playback_done;
    playback_done_reg[1] <= #1 playback_done_reg[0];
    playback_done_reg[2] <= #1 playback_done_reg[1];
    playback_done_reg[3] <= #1 playback_done_reg[2];
    playback_done_reg[4] <= #1 playback_done_reg[3];
  end
end

reg bram_b_clk_ctrl;   
always @ (posedge bclk or negedge brst)
begin
  if (~brst)
    bram_b_clk_ctrl <= #1 1'b0;
  //else if(playback_done_reg[1])//gyu test
  else if(playback_done_reg[2])//gyu test
    bram_b_clk_ctrl <= #1 1'b0;
  else
    bram_b_clk_ctrl <= #1 pre_playback_data;//gyu glitch     
end

reg tx1_ctrl;
always @ (posedge bclk or negedge brst)
begin
  if (~brst)
    tx1_ctrl <= #1 1'b0;
  else if(playback_done_reg[3])
    tx1_ctrl <= #1 1'b0;
  else
    tx1_ctrl <= #1 pre_playback_data;//gyu glitch     
end

//loading control
reg rdn;
//reg [7:0] rxfn_reg;//gyu 2x
reg [12:0] rxfn_reg;//gyu 2x
reg [15:0] loading_done_reg;
//reg bram_wr_window;//gyu fifo handshake
always @ (posedge aclk or negedge arst)
  if (~arst)
  begin
    rdn            <= #1 1'b1;
//    rxfn_reg       <= #1 8'b0;//gyu 2x
    rxfn_reg       <= #1 13'b0;//gyu 2x
    //bram_wr_window <= #1 1'b0;//gyu fifo handshake
  end
  else if (loading_done_reg[2] && rxfn)
  begin
    rdn            <= #1 1'b1;
//    rxfn_reg       <= #1 8'b0;//gyu 2x
    rxfn_reg       <= #1 13'b0;//gyu 2x
    //bram_wr_window <= #1 1'b0;//gyu fifo handshake
  end  
  else
  begin
    rxfn_reg[0]    <= #1 ~rxfn;
    rxfn_reg[1]    <= #1 rxfn_reg[0];
    rxfn_reg[2]    <= #1 rxfn_reg[1];
    rxfn_reg[3]    <= #1 rxfn_reg[2];
    rxfn_reg[4]    <= #1 rxfn_reg[3];
    rxfn_reg[5]    <= #1 rxfn_reg[4];
    rxfn_reg[6]    <= #1 rxfn_reg[5];
    rxfn_reg[7]    <= #1 rxfn_reg[6];
    rxfn_reg[8]    <= #1 rxfn_reg[7];
    rxfn_reg[9]    <= #1 rxfn_reg[8];
    rxfn_reg[10]   <= #1 rxfn_reg[9];
    rxfn_reg[11]   <= #1 rxfn_reg[10];

    rdn            <= #1 ~rxfn_reg[1];
    //bram_wr_window <= #1 rxfn_reg[5] & rxfn_reg[2];//gyu 2x
    //bram_wr_window <= #1 rxfn_reg[9] & rxfn_reg[2];//gyu 2x//gyu fifo handshake
  end

//wire #1 bram_wr_window = ~(((bram_a_addr == {BRAM_ADDR_WIDTH{1'b0}}) ? (~rxfn_reg[10]) : rxfn) | rdn);//gyu fifo handshake//gyu input issue//gyu 1010 issue
wire #1 bram_wr_window = ~(((bram_a_addr == {BRAM_ADDR_WIDTH{1'b0}}) ? (~rxfn_reg[11]) : rxfn) | rdn);//gyu fifo handshake//gyu input issue//gyu 1010 issue

reg bram_wr_stop;
always @ (posedge aclk or negedge arst)
begin
  if(~arst)
  begin
    loading_done_reg    <= #1 16'b0;
    //early_playback_data <= #1 1'b0;//gyu configuration during DAC operation
    bram_wr_stop        <= #1 1'b0;
  end  
  else
  begin
    loading_done_reg[0]  <= #1 loading_done;
    loading_done_reg[1]  <= #1 loading_done_reg[0];
    loading_done_reg[2]  <= #1 loading_done_reg[1];
    loading_done_reg[3]  <= #1 loading_done_reg[2];
    loading_done_reg[4]  <= #1 loading_done_reg[3];
    loading_done_reg[5]  <= #1 loading_done_reg[4];
    loading_done_reg[6]  <= #1 loading_done_reg[5];
    loading_done_reg[7]  <= #1 loading_done_reg[6];
    loading_done_reg[8]  <= #1 loading_done_reg[7];
    loading_done_reg[9]  <= #1 loading_done_reg[8];
    loading_done_reg[10] <= #1 loading_done_reg[9];
    loading_done_reg[11] <= #1 loading_done_reg[10];
    loading_done_reg[12] <= #1 loading_done_reg[11];
    loading_done_reg[13] <= #1 loading_done_reg[12];
    loading_done_reg[14] <= #1 loading_done_reg[13];
    loading_done_reg[15] <= #1 loading_done_reg[14];
    //early_playback_data <= #1 loading_done_reg[7];//gyu configuration during DAC operation
    //bram_wr_stop        <= #1 loading_done_reg[2];//gyu 2x
    //bram_wr_stop        <= #1 loading_done_reg[6];//gyu 2x//gyu glitch
    //bram_wr_stop        <= #1 loading_done_reg[7];//gyu 2x//gyu glitch
    bram_wr_stop        <= #1 loading_done_reg[8];//gyu 2x//gyu glitch//gyu more 8     
  end
end

always @ (posedge aclk or negedge arst)
begin
  if(~arst)
    early_playback_data <= #1 1'b0;
  else if (loading_done_reg[15])//gyu configuration during DAC operation
    early_playback_data <= #1 1'b1;
end

reg [1:0] catch_early_reg;
always @ (posedge bclk or negedge brst)
  if(~brst)
    catch_early_reg     <= #1 2'b0;
   else
   begin
     catch_early_reg[0] <= #1 early_playback_data;
     catch_early_reg[1] <= #1 catch_early_reg[0];
   end 
    
always @ (posedge bclk or negedge brst)
  if(~brst)
  begin
    pre_playback_data   <= #1 1'b0;
    playback_data       <= #1 1'b0;
  end  
  else if (catch_early_reg[1])
  begin
    pre_playback_data   <= #1 1'b1;
    playback_data       <= #1 pre_playback_data;
  end      
    
endmodule

