/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module top_FPGA
(
//gyu spi master input from FDTI
    sck,
    scs,
    mosi,
    miso,
  
//gyu spi slave output to ADC
    adcck,
    adccs,
    adcdi,
    adcdo,

  // ----------------------------------------------------
  // I2C master interface
  // ----------------------------------------------------
    ssda,
    sscl, 

  // ----------------------------------------------------
  // System Clock 
  // ----------------------------------------------------
    SYSCLK_p,
    SYSCLK_n,
    
  // ----------------------------------------------------
  // LCD interface 
  // ----------------------------------------------------

    lcd_d_cn,
    lcd_wen,
    lcd_csn,
    lcd_data,
    
  // ----------------------------------------------------
  // FTDI Sync FIFO interface 
  // ----------------------------------------------------
    fifoclk,
    rxfn,
    fifo_data_in,
    rdn,
    oen,
    
  // ----------------------------------------------------
  // General Control and Jumper 
  // ----------------------------------------------------

    MPI2_RESET,
    
    actpls,
    hostrw,
    adflag,
    cfgrst,
    adport,
    
    //dacpw,//gyu test
    
  // ----------------------------------------------------
  // 2000 Interface
  // ---------------------------------------------------- 
  
    DDCK_p,
    DDCK_n,
    
    DA1_p,
    DA1_n,
    DA2_p,
    DA2_n,
                
    DCLK_p,
    DCLK_n
     
);

`include "/home/gyu/p2000_081814/src/fpga/char_define.h"

parameter buf_2MSPS                 =  4'b0111;
parameter buf_1MSPS                 =  4'b0110; 
parameter buf_512KSPS               =  4'b0101;
parameter buf_256KSPS               =  4'b0100;
parameter buf_128KSPS               =  4'b0011;
parameter buf_64KSPS                =  4'b0010; 
parameter buf_32KSPS                =  4'b0001;
parameter buf_16KSPS                =  4'b0000;

parameter BRAM_A_ADDR_WIDTH         =  17;
parameter BRAM_B_ADDR_WIDTH         =  15;

//***********************************Ports Declaration*******************************

//gyu spi master input from FDTI
  input                   sck;
  input                   scs;
  input                   mosi;
  output                  miso;
  
//gyu spi slave output to ADC
  output                  adcck;
  output                  adccs;
  output                  adcdi;
  input                   adcdo;

  // ----------------------------------------------------
  // I2C master interface
  // ----------------------------------------------------
inout       ssda;
output      sscl; 


  // ----------------------------------------------------
  // System Clock 
  // ----------------------------------------------------
input       SYSCLK_p;
input       SYSCLK_n;
    
  // ----------------------------------------------------
  // LCD interface 
  // ----------------------------------------------------

output                    lcd_d_cn;   //LCD Data/Command select
output                    lcd_wen;    //LCD Read Write select
output                    lcd_csn;    //LCD Chip select
inout    [7:4]            lcd_data;   //LCD Data bus
 
  // ----------------------------------------------------
  // FTDI Sync FIFO interface 
  // ----------------------------------------------------
input       fifoclk;  
input       rxfn;
input [7:0] fifo_data_in;
output      rdn;
output      oen;

  // ----------------------------------------------------
  // General Control and Jumper 
  // ----------------------------------------------------
input         MPI2_RESET;  
  
input         actpls;
input         hostrw;
input         adflag;
input         cfgrst;
inout [7:0]   adport;

//output        dacpw;//gyu test    
  // ----------------------------------------------------
  // 2000 Interface
  // ----------------------------------------------------
input    DDCK_p;
input    DDCK_n;

output [15:0] DA1_p;
output [15:0] DA1_n;
output [15:0] DA2_p;
output [15:0] DA2_n;
                
output DCLK_p;
output DCLK_n;
     
//************************** Register Declarations ****************************

//**************************** Wire Declarations ******************************
wire dclk;
      
//**************************** Main Body of Code *******************************

//spi    
assign      #4 adcck           = sck;
assign      #4 adccs           = scs;
assign      #4 adcdi           = mosi;
assign      #4 miso            = adcdo;

  // ----------------------------------------------------
  // I2C master interface
  // ----------------------------------------------------
wire        SDA_SET, SDA_DET, SCL_SET;
assign      #4.5  ssda         = (~SDA_SET) ? 1'b0 : 1'bz;
assign      #1    SDA_DET      = ssda;

assign      #4.5  sscl         = (~SCL_SET) ? 1'b0 : 1'bz;  



  // ----------------------------------------------------
  // 2000 Interface
  // ----------------------------------------------------

    OBUFDS OBUFDS_dclk
    (
        .O(DCLK_p),
        .OB(DCLK_n),
        .I(dclk)
    );
     
    
wire othbclk;
wire othaclk;
wire prebram_b_clk;
wire bram_a_clk;
wire sclk;
wire pll_locked0;
wire pll_locked1;
wire pll_locked2;

(* DONT_TOUCH *)reg [4:0] gtp_rst_reg;
always @ (posedge sclk or negedge pll_locked0)
  if (~pll_locked0)
    gtp_rst_reg    <= #1 5'b0;
  else
  begin
    gtp_rst_reg[0] <= #1 pll_locked0;
    gtp_rst_reg[1] <= #1 gtp_rst_reg[0];
    gtp_rst_reg[2] <= #1 gtp_rst_reg[1];
    gtp_rst_reg[3] <= #1 gtp_rst_reg[2];
    gtp_rst_reg[4] <= #1 gtp_rst_reg[3];    
  end  

wire  gtp_a_rst;
wire  gtp_b_rst;
wire  CTRLRST_SET;
assign #1 gtp_a_rst = gtp_rst_reg[4] & pll_locked2 & CTRLRST_SET;
assign #1 gtp_b_rst = gtp_rst_reg[4] & pll_locked1 & CTRLRST_SET;
 
clk_wiz_sclk clk_wiz_sclk
 (// Clock in ports
  .clk_in1_p(SYSCLK_p),
  .clk_in1_n(SYSCLK_n),
  // Clock out ports
  .clk_out1(sclk),
  // Status and control signals
  .reset(MPI2_RESET | (~cfgrst)),
  .locked(pll_locked0)
 );

wire tx1_clk, tx2_clk;
wire tx1_div_clk, tx2_div_clk;

clk_wiz_tx_div clk_wiz_tx_div
 (// Clock in ports
  .clk_in1_p(DDCK_p),
  .clk_in1_n(DDCK_n),

  // Clock out ports
  .clk_out1(tx1_clk),
  .clk_out2(tx2_clk),
  .clk_out3(tx1_div_clk),
  .clk_out4(tx2_div_clk),
  
  //.clk_out5(othbclk),//gyu glitch
  //.clk_out6(prebram_b_clk),//gyu glitch
  
  .clk_out5(prebram_b_clk),//gyu glitch
  .clk_out6(othbclk),//gyu glitch

  .clk_out7(dclk),
  
  // Status and control signals
  .reset(~gtp_rst_reg[4]),
  .locked(pll_locked1)
 );


 
clk_wiz_fclk clk_wiz_fclk
 (// Clock in ports
  .clk_in1(fifoclk),
  // Clock out ports
  .clk_out1(bram_a_clk),//gyu rm ctrl
  .clk_out2(othaclk),
  // Status and control signals
  .reset(~gtp_rst_reg[4]),
  .locked(pll_locked2)
 );

(* DONT_TOUCH *)reg tx_io_reset;
(* DONT_TOUCH *)reg [7:0] tx_reset_cnt;
always @ (posedge prebram_b_clk or negedge pll_locked1)//gyu glitch
  if (~pll_locked1)
  begin
    tx_io_reset    <= #1 1'b1;
    tx_reset_cnt   <= #1 8'h0;
  end
  else if (tx_reset_cnt == 8'hff)
  begin 
    tx_io_reset    <= #1 1'b0;
    tx_reset_cnt   <= #1 tx_reset_cnt;
  end
  else
  begin
    tx_io_reset    <= #1 tx_io_reset;
    tx_reset_cnt   <= #1 tx_reset_cnt + 1;
  end
 
wire tx1_ctrl;
wire #1 tx_rst = tx_io_reset | (~tx1_ctrl);
    
wire [127:0] alt_pretest_dword1;
wire [127:0] alt_pretest_dword2;

(* DONT_TOUCH *)reg [127:0] alt_test_dword1, alt_test_dword2;

always @ (posedge prebram_b_clk or posedge tx_rst)
  if (tx_rst)
    alt_test_dword1      <= #1 128'b0;  
  else
    alt_test_dword1      <= #1 {alt_pretest_dword2[111:96],
                                alt_pretest_dword2[79:64],
                                alt_pretest_dword2[47:32],
                                alt_pretest_dword2[15:0],
                                alt_pretest_dword1[111:96],
                                alt_pretest_dword1[79:64],
                                alt_pretest_dword1[47:32],
                                alt_pretest_dword1[15:0]};
                                
always @ (posedge prebram_b_clk or posedge tx_rst)
  if (tx_rst) 
    alt_test_dword2      <= #1 128'b0;
  else                            
    alt_test_dword2      <= #1 {alt_pretest_dword2[127:112],
                                alt_pretest_dword2[95:80],
                                alt_pretest_dword2[63:48],
                                alt_pretest_dword2[31:16],
                                alt_pretest_dword1[127:112],
                                alt_pretest_dword1[95:80],
                                alt_pretest_dword1[63:48],
                                alt_pretest_dword1[31:16]}; 
                         
selectio_tx1 /* #
(
  .SYS_W(16),   // width of the data for the system
  .DEV_W(128)   // width of the data for the device
) */ 
selectio_tx1_1
(
  // From the device out to the system
  .data_out_from_device(alt_test_dword1),       //[DEV_W-1:0]
  .data_out_to_pins_p(DA1_p),                   //[SYS_W-1:0]
  .data_out_to_pins_n(DA1_n),                   //[SYS_W-1:0]

  .clk_in(tx1_clk),                             //Fast clock input from PLL/MMCM
  .clk_div_in(tx1_div_clk),                     //Slow clock input from PLL/MMCM
  .io_reset(tx_rst));
       
selectio_tx1 /* #
(
  .SYS_W(16),   // width of the data for the system
  .DEV_W(128)   // width of the data for the device
) */
selectio_tx1_2
(
  // From the device out to the system
  .data_out_from_device(alt_test_dword2),       //[DEV_W-1:0]
  .data_out_to_pins_p(DA2_p),                   //[SYS_W-1:0]
  .data_out_to_pins_n(DA2_n),                   //[SYS_W-1:0]
   
  .clk_in(tx2_clk),                             //Fast clock input from PLL/MMCM
  .clk_div_in(tx2_div_clk),                     //Slow clock input from PLL/MMCM
  .io_reset(tx_rst));
  

wire bram_b_clk_ctrl;                                    
/* //gyu glitch
wire bram_b_clk;  
BUFGCE BUFGCE_bram_b_clk
(
.CE(bram_b_clk_ctrl),
.I(prebram_b_clk),
.O(bram_b_clk)
);
*/

assign #1 oen = 1'b0;

wire bram_wr_window;
wire bram_wr_stop;      

wire       loading_done;
(* DONT_TOUCH *)reg  [4:0] lastwr_cnt;      
always @ (posedge bram_a_clk or negedge gtp_a_rst)//gyu more 8
  if(~gtp_a_rst)
    lastwr_cnt           <= #1 5'b0;
  else if (loading_done)
    if (lastwr_cnt == 5'h1f)
      lastwr_cnt           <= #1 lastwr_cnt;
    else
      lastwr_cnt           <= #1 lastwr_cnt + 1;
         
//gyu 2x
(* DONT_TOUCH *)reg [7:0] fifo_data_reg0;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg1;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg2;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg3;

(* DONT_TOUCH *)reg [7:0] fifo_data_reg4;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg5;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg6;
(* DONT_TOUCH *)reg [7:0] fifo_data_reg7;
 
wire #1 capture_window = ~(rdn | rxfn | bram_wr_stop);//gyu 1010 issue
always @ (posedge bram_a_clk or negedge gtp_a_rst)
  if(~gtp_a_rst)
  begin
    fifo_data_reg0       <= #1 8'b0;
    fifo_data_reg1       <= #1 8'b0;
    fifo_data_reg2       <= #1 8'b0;
    fifo_data_reg3       <= #1 8'b0;
    
    fifo_data_reg4       <= #1 8'b0;
    fifo_data_reg5       <= #1 8'b0;
    fifo_data_reg6       <= #1 8'b0;
    fifo_data_reg7       <= #1 8'b0;
  end
  else if (lastwr_cnt == 5'h07)//gyu more 8
  begin
    fifo_data_reg0  <= #1 fifo_data_in;
    fifo_data_reg1  <= #1 fifo_data_reg0;
    fifo_data_reg2  <= #1 fifo_data_reg1;
    fifo_data_reg3  <= #1 fifo_data_reg2;
    
    fifo_data_reg4  <= #1 fifo_data_reg3;
    fifo_data_reg5  <= #1 fifo_data_reg4;
    fifo_data_reg6  <= #1 fifo_data_reg5;
    fifo_data_reg7  <= #1 fifo_data_reg6;
  end
  else if ((capture_window) && (lastwr_cnt[4:3] == 2'b0))//gyu 1010 issue//gyu more 8
//  else//gyu 1010 issue
  begin
    fifo_data_reg0  <= #1 fifo_data_in;
    fifo_data_reg1  <= #1 fifo_data_reg0;
    fifo_data_reg2  <= #1 fifo_data_reg1;
    fifo_data_reg3  <= #1 fifo_data_reg2;
    
    fifo_data_reg4  <= #1 fifo_data_reg3;
    fifo_data_reg5  <= #1 fifo_data_reg4;
    fifo_data_reg6  <= #1 fifo_data_reg5;
    fifo_data_reg7  <= #1 fifo_data_reg6;
  end

(* DONT_TOUCH *)reg bram_a_wren;
(* DONT_TOUCH *)reg loading_data;//gyu glitch
(* DONT_TOUCH *)reg [2:0] fifo81_cnt;

//wire [15 : 0] bram_a_addr;//gyu 2x
wire [16 : 0] bram_a_addr;//gyu 2x
//wire [12 : 0] bram_b_addr;//gyu 2x
wire [14 : 0] bram_b_addr;//gyu 2x
wire       MEMWR_SET;//gyu nowr
      
always @ (posedge bram_a_clk or negedge gtp_a_rst)
  if(~gtp_a_rst)
  begin
    fifo81_cnt           <= #1 3'b0;
    bram_a_wren          <= #1 1'b0;
    loading_data         <= #1 1'b0;//gyu glitch
  end
  else if (lastwr_cnt == 5'h0d)//gyu more 8
    bram_a_wren            <= #1 1'b1;
  else if ((bram_wr_window) && (!bram_wr_stop) && (lastwr_cnt[4:3] == 2'b0))//gyu more 8
  begin
    fifo81_cnt              <= #1 fifo81_cnt + 3'b01;
    if (fifo81_cnt == 3'b000)
    begin                  
      bram_a_wren            <= #1 1'b0;
      loading_data           <= #1 1'b0;//gyu glitch
    end
    else if (fifo81_cnt == 3'b101) //gyu nowr//gyu glitch
    begin
    //else if ((fifo81_cnt == 3'b110) && (MEMWR_SET)) //gyu nowr
      bram_a_wren            <= #1 1'b1;
      loading_data           <= #1 1'b0;//gyu glitch
    end  
    else if (fifo81_cnt == 3'b110) //gyu nowr//gyu glitch
    begin
      bram_a_wren            <= #1 1'b0;
      loading_data           <= #1 1'b1;//gyu glitch
    end  
    else
    begin
      bram_a_wren            <= #1 1'b0;
      loading_data           <= #1 1'b0;//gyu glitch  
    end  
  end
  else
  begin
    bram_a_wren         <= #1 1'b0;
    loading_data        <= #1 1'b0;//gyu glitch  
  end
  
(* DONT_TOUCH *)reg [63:0] fifo_data_reg;
always @ (posedge bram_a_clk or negedge gtp_a_rst)
  if(~gtp_a_rst)
    fifo_data_reg        <= #1 64'b0;
  else if (lastwr_cnt == 5'h08)//gyu more 8
    fifo_data_reg        <= #1 {fifo_data_reg1, fifo_data_reg0, fifo_data_reg3, fifo_data_reg2,
                                fifo_data_reg5, fifo_data_reg4, fifo_data_reg7, fifo_data_reg6}; 
  else if ((bram_wr_window) && (!bram_wr_stop) && (fifo81_cnt == 3'b000) && (!lastwr_cnt[4]))
    fifo_data_reg        <= #1 {fifo_data_reg1, fifo_data_reg0, fifo_data_reg3, fifo_data_reg2,
                                fifo_data_reg5, fifo_data_reg4, fifo_data_reg7, fifo_data_reg6};     

                                
                            
  
blk_mem_gen_1 blk_mem_gen_1(
  .wea(bram_a_wren),
  .addra(bram_a_addr),
  .clka(bram_a_clk),
  .dina(fifo_data_reg),
  
  .addrb(bram_b_addr),
  .clkb(prebram_b_clk),//gyu glitch
  .enb(bram_b_clk_ctrl),//gyu glitch
  .doutb({alt_pretest_dword2, alt_pretest_dword1})
);
//gyu 2x
                        
wire       MODE_SET;
wire [3:0] MEMSIZE_SET;
                     
wire  [7:0] adport_in; 
wire  [7:0] adport_out;                
assign #4.5 adport    = (hostrw) ? adport_out : 8'hz;
assign #1   adport_in =  adport;

wire dacpw;//gyu test
ftdi_cfg ftdi_cfg
(
//i2c write register    
    .SDA_SET(SDA_SET),
    .SCL_SET(SCL_SET),
//i2c read register            
    .SDA_DET(SDA_DET),    

    .MEMWR_SET(MEMWR_SET),//gyu nowr
    .MODE_SET(MODE_SET),
    .MEMSIZE_SET(MEMSIZE_SET),
    .DACPW_SET(dacpw),
    .CTRLRST_SET(CTRLRST_SET),
    
    .PLLLK0_DET(pll_locked0),
    .PLLLK1_DET(pll_locked1),
    .PLLLK2_DET(pll_locked2),
    
    .actpls(actpls),
    .hostrw(hostrw),
    .adflag(adflag),
    .rst(gtp_rst_reg[4]),
    .selfclk(sclk),
    .pd_rst(gtp_rst_reg[4]),//keep clock on anyway gyu
    .adport_in(adport_in), 
    .adport_out(adport_out)
       
);



                      

//reg [9:0]  BUFFER_B_ADDR_CNT;//gyu 2x
reg [11:0] BUFFER_B_ADDR_CNT;//gyu 2x
//reg [12:0] BUFFER_A_ADDR_CNT;//gyu 2x
reg [13:0] BUFFER_A_ADDR_CNT;//gyu 2x
reg [55:0] BUFSIZE_DSPLY;
always @ (MEMSIZE_SET)
  casex(MEMSIZE_SET)
  // MEMSIZE_512KSPS
  buf_512KSPS : begin BUFFER_B_ADDR_CNT <=  #1 12'hfff;//gyu 2x
                      BUFFER_A_ADDR_CNT <=  #1 14'h3fff;
                      BUFSIZE_DSPLY   <=  #1 {_5, _1, _2, _K, _S, _P, s_};
                end
  // MEMSIZE_256KSPS
  buf_256KSPS : begin BUFFER_B_ADDR_CNT <=  #1 12'h7ff;//gyu 2x
                      BUFFER_A_ADDR_CNT <=  #1 14'h1fff;
                      BUFSIZE_DSPLY   <=  #1 {_2, _5, _6, _K, _S, _P, s_};
                end
  // MEMSIZE_128KSPS
  buf_128KSPS : begin BUFFER_B_ADDR_CNT <=  #1 12'h3ff;
                      BUFFER_A_ADDR_CNT <=  #1 14'h0fff;
                      BUFSIZE_DSPLY   <=  #1 {_1, _2, _8, _K, _S, _P, s_};
                end
  // MEMSIZE_64KSPS
  buf_64KSPS  : begin BUFFER_B_ADDR_CNT <=  #1 12'h1ff;
                      BUFFER_A_ADDR_CNT <=  #1 14'h07ff;
                      BUFSIZE_DSPLY   <=  #1 {_sp, _6, _4, _K, _S, _P, s_};
                end  
  // MEMSIZE_32KSPS
  buf_32KSPS  : begin BUFFER_B_ADDR_CNT <=  #1 12'h0ff;
                      BUFFER_A_ADDR_CNT <=  #1 14'h03ff;
                      BUFSIZE_DSPLY   <=  #1 {_sp, _3, _2, _K, _S, _P, s_};
                end  
  // MEMSIZE_16KSPS
  buf_16KSPS  : begin BUFFER_B_ADDR_CNT <=  #1 12'h07f;
                      BUFFER_A_ADDR_CNT <=  #1 14'h01ff;
                      BUFSIZE_DSPLY   <=  #1 {_sp, _1, _6, _K, _S, _P, s_};
                end               
  // MEMSIZE_16KSPS
  default     : begin BUFFER_B_ADDR_CNT <=  #1 12'h07f;
                      BUFFER_A_ADDR_CNT <=  #1 14'h01ff;
                      BUFSIZE_DSPLY   <=  #1 {_sp, _1, _6, _K, _S, _P, s_};
                end
  endcase

reg [15:0] MODE_DSPLY;  
always @ (MODE_SET)
  if(MODE_SET)
    MODE_DSPLY <= #1 {_S, _F};
  else
    MODE_DSPLY <= #1 {_M, _F};
    
reg [23:0] STATUS_DSPLY;  
always @ (gtp_rst_reg[4], pll_locked2, pll_locked1, pll_locked0)
  casex({gtp_rst_reg[4], pll_locked2, pll_locked1, pll_locked0})
  4'b0xxx : STATUS_DSPLY <= #1 {_R, _S, _T};
  4'b10xx : STATUS_DSPLY <= #1 {_R, _S, _T};
  4'b110x : STATUS_DSPLY <= #1 {_C, _F, _G};
  4'b1110 : STATUS_DSPLY <= #1 {_C, _L, _K};
  4'b1111 : STATUS_DSPLY <= #1 {_R, _U, _N};
  default : STATUS_DSPLY <= #1 {_C, _F, _G};
  endcase
  
                           
wire         playback_done;
wire         playback_data;
altbram_addr #
(
.BRAM_ADDR_WIDTH(BRAM_B_ADDR_WIDTH)
)
altbram_b_addr
(
//.addr_trigger(bram_b_clk),//gyu glitch 
.addr_trigger(prebram_b_clk),//gyu glitch
.addr(bram_b_addr),

.buf_mode(MODE_SET),
.cnt_done(playback_done),
.cnt_data(playback_data),

//.BUFFER_ADDR_CNT({3'b0, BUFFER_B_ADDR_CNT}),//gyu 2x
.BUFFER_ADDR_CNT(BUFFER_B_ADDR_CNT),//gyu 2x

.rst(gtp_b_rst));



altbram_addr #
(
.BRAM_ADDR_WIDTH(BRAM_A_ADDR_WIDTH)
)
altbram_a_addr
(
.addr_trigger(bram_a_clk), 
.addr(bram_a_addr),

.buf_mode(1'b1),
.cnt_done(loading_done),
.cnt_data(loading_data),

.BUFFER_ADDR_CNT(BUFFER_A_ADDR_CNT),

.rst(gtp_a_rst));

  
altbram_ctrl #
(
.BRAM_ADDR_WIDTH(BRAM_A_ADDR_WIDTH)
)
altbram_ctrl
(
//playback control
.playback_data(playback_data),
.playback_done(playback_done),
.bclk(prebram_b_clk),//gyu glitch
.bram_b_clk_ctrl(bram_b_clk_ctrl),
.tx1_ctrl(tx1_ctrl),

//loading control
.rxfn(rxfn),
.loading_done(loading_done),
.rdn(rdn),
.aclk(bram_a_clk),
.bram_wr_window(bram_wr_window),
.bram_a_addr(bram_a_addr),//gyu fifo handshake
.bram_wr_stop(bram_wr_stop),
.arst(gtp_a_rst),
.brst(gtp_b_rst));
  

wire rsgen;
wire rwgen;
wire egen;
wire [7:0]lcdgen;
wire [3:0] float_wire;
 
lcdrw_ctrl lcdrw_ctrl(
.rst(pll_locked0),//fpga rst status check gyu
.clk(sclk),

.rsin(rsgen),
.rwin(rwgen),
.ein(egen),
.lcdin(lcdgen),

.rsout(lcd_d_cn),
.rwout(lcd_wen),
.eout(lcd_csn),

.lcdda({lcd_data, float_wire})
  );
  
lcdfl_ctrl lcdfl_ctrl(
.rst(pll_locked0),//fpga rst status check gyu
.clk(sclk),

.rsgen(rsgen),
.rwgen(rwgen),
.egen(egen),
.lcdgen(lcdgen),

.lcd_up_line({STATUS_TITLE_DSPLY, STATUS_DSPLY, _sp, MODE_TITLE_DSPLY, MODE_DSPLY}),
              
.lcd_dn_line({BUFSIZE_TITLE_DSPLY, BUFSIZE_DSPLY}),

.lcd_busy(lcd_data[7])
  );

endmodule



