/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/
module ftdi_cfg
(
//i2c write register    
    SDA_SET,
    SCL_SET,
//i2c read register            
    SDA_DET,      

    MEMWR_SET,//gyu nowr
    MODE_SET,
    MEMSIZE_SET,
    
    DACPW_SET,
  
    CTRLRST_SET,
        
    PLLLK0_DET,
    PLLLK1_DET,
    PLLLK2_DET,

    actpls,
    hostrw,
    adflag,
    rst,
    selfclk,    
    pd_rst,//keep clock on anyway gyu
    
    adport_in, 
    adport_out
       
);

parameter id_value                  =  8'h20;

parameter multi_frame               =  1'h0;
parameter single_frame              =  1'h1;
parameter dac_shutdown              =  1'b0;




parameter buf_2MSPS                 =  4'b0111;
parameter buf_1MSPS                 =  4'b0110; 
parameter buf_512KSPS               =  4'b0101;
parameter buf_256KSPS               =  4'b0100;
parameter buf_128KSPS               =  4'b0011;
parameter buf_64KSPS                =  4'b0010; 
parameter buf_32KSPS                =  4'b0001;
parameter buf_16KSPS                =  4'b0000;


//i2c write register    
output        SDA_SET;
output        SCL_SET;
//i2c read register            
input           SDA_DET;  

output        MEMWR_SET;//gyu nowr
output        MODE_SET;
output [3:0]  MEMSIZE_SET;
output        DACPW_SET;

output        CTRLRST_SET;

input         PLLLK0_DET;
input         PLLLK1_DET;
input         PLLLK2_DET;

input         actpls;
input         hostrw;
input         adflag;
input         rst;
input         selfclk;
input         pd_rst;//keep clock on anyway gyu

input   [7:0] adport_in; 
output  [7:0] adport_out;

//i2c write register    
reg       SDA_SET;
reg       SCL_SET;

reg [7:0] FPGA_ID_REG;
       
//reg [7:0] FPGA_CTRL_REG;
reg       MEMWR_SET;//gyu nowr
reg       MODE_SET;
reg [3:0] MEMSIZE_SET;
reg       DACPW_SET;

//capture reset register
reg       CTRLRST_SET;

//reg [7:0] FPGA_STATUS_REG;
//input     PLLLK0_DET;
//input     PLLLK1_DET;
//input     PLLLK2_DET;

reg [7:0] adport_out;


wire FPGA_ID_SEL;       
wire FPGA_CTRL_SEL;
wire FPGA_STATUS_SEL;

wire DAC_CTRL_SEL;
wire CAP_RST_SEL;

wire I2C_ACSS_SEL;

add_dec add_dec
(
.actpls(actpls), 
.adflag(adflag),
.addr(adport_in), 
.rst(rst),

.FPGA_ID_SEL(FPGA_ID_SEL),      
.FPGA_CTRL_SEL(FPGA_CTRL_SEL),
.FPGA_STATUS_SEL(FPGA_STATUS_SEL),
.DAC_CTRL_SEL(DAC_CTRL_SEL),
.CAP_RST_SEL(CAP_RST_SEL),

.I2C_ACSS_SEL(I2C_ACSS_SEL)
);

  // ----------------------------------------------------
  // Register Read by Host
  // ----------------------------------------------------

always @ (posedge actpls or negedge rst)
  if (~rst)
  begin
    adport_out  <= #1 8'h0;
  end
  else if (hostrw)
  begin
    if (FPGA_ID_SEL)
       adport_out  <= #1  FPGA_ID_REG;
    //else if (FPGA_CTRL_SEL)
    else if (FPGA_STATUS_SEL)
       adport_out  <= #1  {4'b0, PLLLK2_DET, PLLLK1_DET, PLLLK0_DET, 1'b0};
    else if (I2C_ACSS_SEL)            
       adport_out  <= #1  {7'b0, SDA_DET};          
  end
  
  // ----------------------------------------------------
  // Register Write by Host
  // ----------------------------------------------------

always @ (posedge actpls or negedge rst)
  if (~rst)
  begin
    FPGA_ID_REG     <= #1 id_value;
    //i2c write register    
    SDA_SET         <= #1 1'b1;
    SCL_SET         <= #1 1'b1;
    MEMWR_SET       <= #1 1'b0;//gyu nowr
    MODE_SET        <= #1 single_frame;
    MEMSIZE_SET     <= #1 buf_16KSPS;
    //DACPW_SET       <= #1 dac_shutdown;//keep clock on anyway gyu
  end
  else if ((~hostrw) & (~adflag))
  begin
    if (FPGA_ID_SEL)
      FPGA_ID_REG   <= #1 adport_in;
    else if (FPGA_CTRL_SEL)
    begin
      MEMSIZE_SET   <= #1 adport_in[7:4];
      MEMWR_SET     <= #1 adport_in[1];//gyu nowr
      MODE_SET      <= #1 adport_in[0];
    end   
    //else if (FPGA_STATUS_SEL)
    //else if (DAC_CTRL_SEL)//keep clock on anyway gyu
      //DACPW_SET     <= #1 adport_in[0];//keep clock on anyway gyu
    else if (I2C_ACSS_SEL)
    begin    
      SCL_SET       <= #1 adport_in[1];
      SDA_SET       <= #1 adport_in[0];
    end  
  end
  
always @ (posedge actpls or negedge pd_rst)//keep clock on anyway gyu
  if (~pd_rst)
    DACPW_SET     <= #1 dac_shutdown;
  else if ((~hostrw) & (~adflag) & DAC_CTRL_SEL)
    DACPW_SET     <= #1 adport_in[0];

    
//capure reset register self clean bit
reg [3:0] CTRLRST_SET_selfcnt;    
always @ (posedge selfclk or negedge rst)
  if (~rst)
    CTRLRST_SET_selfcnt     <= #1 4'hf;
  else
  begin
    CTRLRST_SET_selfcnt[0]  <= #1 CTRLRST_SET;
    CTRLRST_SET_selfcnt[1]  <= #1 CTRLRST_SET_selfcnt[0];
    CTRLRST_SET_selfcnt[2]  <= #1 CTRLRST_SET_selfcnt[1];
    CTRLRST_SET_selfcnt[3]  <= #1 CTRLRST_SET_selfcnt[2]; 
  end

wire CTRLRST_SET_rst = rst & (~(CTRLRST_SET_selfcnt == 4'b0));   
always @ (posedge actpls or negedge CTRLRST_SET_rst)
  if (~CTRLRST_SET_rst)
    CTRLRST_SET        <= #1 1'b1;   
  else if ((~hostrw) & (~adflag) & (CAP_RST_SEL))
    CTRLRST_SET        <= #1 ~adport_in[0];

    
endmodule
