/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module lcdfl_ctrl (
rst,
clk,

rsgen,
rwgen,
egen,
lcdgen,

lcd_up_line,
lcd_dn_line,

lcd_busy
);

input rst;
input clk;

output rsgen;
output rwgen;
output egen;
output [7:0]lcdgen;

input [127:0]lcd_up_line;
input [127:0]lcd_dn_line;

input lcd_busy;

`include "/home/gyu/p2000_081814/src/fpga/char_define.h"

parameter INT0_STAT     = 3'h0;
parameter INT1_STAT     = 3'h1;
parameter INT2_STAT     = 3'h2;
parameter DPCH_STAT     = 3'h3;
parameter SETA_STAT     = 3'h4;

parameter DISPLY_ON     = 8'h0c;
parameter DISPLY_2L     = 8'h38;
parameter DISPLY_CR     = 8'h01;
parameter ADDSET_1L     = 8'h80;
parameter ADDSET_2L     = 8'hc0;

reg [2:0] next_state;
reg [2:0] state_cnt;
reg [4:0] char_cnt;
reg [7:0] char_dsp;
reg rsgen;
reg rwgen;
reg egen;
reg [7:0]lcdgen;
always @ (posedge clk or negedge rst)
  if(~rst)
  begin
    next_state <= #1 INT0_STAT;
    state_cnt  <= #1 3'b0;
    char_cnt   <= #1 5'b0;
    rsgen      <= #1 1'b0;
    rwgen      <= #1 1'b1;
    egen       <= #1 1'b0;
    lcdgen     <= #1 8'b0;
  end
  else
  begin
    state_cnt  <= #1 state_cnt + 3'b001;
    case(next_state)
     
    INT0_STAT : begin
                    if(state_cnt == 3'h0)
                    begin//Checking
                        //next_state;
                        //char_cnt;
                        rsgen      <= #1 1'b0;
                        rwgen      <= #1 1'b1;
                        egen       <= #1 1'b1;
                        //lcdgen;
                    end
                    else if(state_cnt == 3'h1)
                        egen       <= #1 1'b0;
                    else if(state_cnt == 3'h4)
                    begin//setting
                        if(~lcd_busy)
                        begin
                            //next_state;
                            //char_cnt;
                            rsgen      <= #1 1'b0;
                            rwgen      <= #1 1'b0;
                            egen       <= #1 1'b1;
                            lcdgen     <= #1 DISPLY_CR;
                        end
                    end   
                    else if(state_cnt == 3'h5)
                    begin
                        if(egen)
                        begin 
                            next_state <= #1 INT1_STAT;
                            egen       <= #1 1'b0;
                        end  
                    end
                end
  
    INT1_STAT : begin
                    if(state_cnt == 3'h0)
                    begin//Checking
                        //next_state;
                        //char_cnt;
                        rsgen      <= #1 1'b0;
                        rwgen      <= #1 1'b1;
                        egen       <= #1 1'b1;
                        //lcdgen;
                    end
                    else if(state_cnt == 3'h1)
                        egen       <= #1 1'b0;
                    else if(state_cnt == 3'h4)
                    begin//setting
                        if(~lcd_busy)
                        begin
                            //next_state;
                            //char_cnt;
                            rsgen      <= #1 1'b0;
                            rwgen      <= #1 1'b0;
                            egen       <= #1 1'b1;
                            lcdgen     <= #1 DISPLY_2L;
                        end
                    end   
                    else if(state_cnt == 3'h5)
                    begin
                        if(egen)
                        begin 
                            next_state <= #1 INT2_STAT;
                            egen       <= #1 1'b0;
                        end  
                    end
                end
        
    INT2_STAT : begin
                    if(state_cnt == 3'h0)
                    begin//Checking
                        //next_state;
                        //char_cnt;
                        rsgen      <= #1 1'b0;
                        rwgen      <= #1 1'b1;
                        egen       <= #1 1'b1;
                        //lcdgen;
                    end
                    else if(state_cnt == 3'h1)
                        egen       <= #1 1'b0;
                    else if(state_cnt == 3'h4)
                    begin//setting
                        if(~lcd_busy)
                        begin
                            //next_state;
                            //char_cnt;
                            rsgen      <= #1 1'b0;
                            rwgen      <= #1 1'b0;
                            egen       <= #1 1'b1;
                            lcdgen     <= #1 DISPLY_ON;
                        end
                    end   
                    else if(state_cnt == 3'h5)
                    begin
                        if(egen)
                        begin 
                            next_state <= #1 DPCH_STAT;
                            egen       <= #1 1'b0;
                        end  
                    end
                end
                
    SETA_STAT : begin
                    if(state_cnt == 3'h0)
                    begin//Checking
                        //next_state;
                        //char_cnt;
                        rsgen      <= #1 1'b0;
                        rwgen      <= #1 1'b1;
                        egen       <= #1 1'b1;
                        //lcdgen;
                    end
                    else if(state_cnt == 3'h1)
                        egen       <= #1 1'b0;
                    else if(state_cnt == 3'h4)
                    begin//setting
                        if(~lcd_busy)
                        begin
                            //next_state;
                            //char_cnt;
                            rsgen      <= #1 1'b0;
                            rwgen      <= #1 1'b0;
                            egen       <= #1 1'b1;
                            if(char_cnt == 5'h10)
                              lcdgen     <= #1 ADDSET_2L;
                            else
                              lcdgen     <= #1 ADDSET_1L;  
                        end
                    end   
                    else if(state_cnt == 3'h5)
                    begin
                        if(egen)
                        begin 
                            next_state <= #1 DPCH_STAT;
                            egen       <= #1 1'b0;
                        end  
                    end
                end
           
    DPCH_STAT : begin
                    if(state_cnt == 3'h0)
                    begin//Checking
                        //next_state;
                        //char_cnt;
                        rsgen      <= #1 1'b0;
                        rwgen      <= #1 1'b1;
                        egen       <= #1 1'b1;
                        //lcdgen;
                    end
                    else if(state_cnt == 3'h1)
                        egen       <= #1 1'b0;
                    else if(state_cnt == 3'h4)
                    begin//setting
                        if(~lcd_busy)
                        begin
                            //next_state;
                            char_cnt   <= #1 char_cnt + 5'h01;
                            rsgen      <= #1 1'b1;
                            rwgen      <= #1 1'b0;
                            egen       <= #1 1'b1;
                            lcdgen     <= #1 char_dsp;
                            
                        end
                    end   
                    else if(state_cnt == 3'h5)
                    begin
                        if(egen)
                        begin
                            if((char_cnt == 5'h00) || (char_cnt == 5'h10))
                                next_state <= #1 SETA_STAT;
                            else    
                                next_state <= #1 DPCH_STAT;
                            egen       <= #1 1'b0;
                        end  
                    end
                end
                
      default : begin
                  next_state <= #1 INT0_STAT;
                  state_cnt  <= #1 3'b0;
                  char_cnt   <= #1 5'b0;
                  rsgen      <= #1 1'b0;
                  rwgen      <= #1 1'b1;
                  egen       <= #1 1'b0;
                  lcdgen     <= #1 8'b0;
                end
       
      endcase                    
                    
  end 
 
   
always @ (posedge clk or negedge rst)
  if(~rst)
    char_dsp <= #1 _sp;
  else
    case(char_cnt)  
    5'h00   : char_dsp <= #1 lcd_up_line[127:120];
    5'h01   : char_dsp <= #1 lcd_up_line[119:112]; 
    5'h02   : char_dsp <= #1 lcd_up_line[111:104];
    5'h03   : char_dsp <= #1 lcd_up_line[103:96];
    5'h04   : char_dsp <= #1 lcd_up_line[95:88];
    5'h05   : char_dsp <= #1 lcd_up_line[87:80]; 
    5'h06   : char_dsp <= #1 lcd_up_line[79:72];
    5'h07   : char_dsp <= #1 lcd_up_line[71:64];
    5'h08   : char_dsp <= #1 lcd_up_line[63:56];
    5'h09   : char_dsp <= #1 lcd_up_line[55:48]; 
    5'h0a   : char_dsp <= #1 lcd_up_line[47:40];
    5'h0b   : char_dsp <= #1 lcd_up_line[39:32];
    5'h0c   : char_dsp <= #1 lcd_up_line[31:24];
    5'h0d   : char_dsp <= #1 lcd_up_line[23:16]; 
    5'h0e   : char_dsp <= #1 lcd_up_line[15:8];
    5'h0f   : char_dsp <= #1 lcd_up_line[7:0];
    5'h10   : char_dsp <= #1 lcd_dn_line[127:120];
    5'h11   : char_dsp <= #1 lcd_dn_line[119:112]; 
    5'h12   : char_dsp <= #1 lcd_dn_line[111:104];
    5'h13   : char_dsp <= #1 lcd_dn_line[103:96];
    5'h14   : char_dsp <= #1 lcd_dn_line[95:88];
    5'h15   : char_dsp <= #1 lcd_dn_line[87:80]; 
    5'h16   : char_dsp <= #1 lcd_dn_line[79:72];
    5'h17   : char_dsp <= #1 lcd_dn_line[71:64];
    5'h18   : char_dsp <= #1 lcd_dn_line[63:56];
    5'h19   : char_dsp <= #1 lcd_dn_line[55:48]; 
    5'h1a   : char_dsp <= #1 lcd_dn_line[47:40];
    5'h1b   : char_dsp <= #1 lcd_dn_line[39:32];
    5'h1c   : char_dsp <= #1 lcd_dn_line[31:24];
    5'h1d   : char_dsp <= #1 lcd_dn_line[23:16]; 
    5'h1e   : char_dsp <= #1 lcd_dn_line[15:8];
    5'h1f   : char_dsp <= #1 lcd_dn_line[7:0];
    default : char_dsp <= #1 _sp;
     
    endcase
    
    
    
endmodule
