/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module lcdrw_ctrl(
rst,
clk,

rsin,
rwin,
ein,
lcdin,

rsout,
rwout,
eout,

lcdda
  );
  
input rst;
input clk;

input rsin;
input rwin;
input ein;
input  [7:0] lcdin;

output rsout;
output rwout;
output eout;

inout [7:0] lcdda;

reg rsout;
reg rwout;
reg eout;
reg [1:0] fl_cnt;

assign #1 lcdda = (rwout) ? 8'bz : lcdin;

always @ (posedge clk or negedge rst)
if(~rst)
  fl_cnt <= #1 2'b0;
else if ((fl_cnt == 2'b0) && (ein)) 
  fl_cnt <= #1 fl_cnt + 2'b01;
else if ((fl_cnt != 2'b0) && (!ein)) 
  fl_cnt <= #1 fl_cnt + 2'b01;
  
always @ (posedge clk or negedge rst)
if(~rst)
begin
  rsout    <= #1 1'b0;
  rwout    <= #1 1'b1;
  eout     <= #1 1'b0;
end
else
  case(fl_cnt)
  2'b00  : begin
            rsout    <= #1 rsin;
            rwout    <= #1 rwin;
            eout     <= #1 1'b0;
            end
  2'b01  : begin
            rsout    <= #1 rsin;
            rwout    <= #1 rwin;
            eout     <= #1 1'b1;
            end
  2'b10  : begin
            rsout    <= #1 rsin;
            rwout    <= #1 rwin;
            eout     <= #1 1'b1;
            end
  2'b11  : begin
            rsout    <= #1 rsin;
            rwout    <= #1 rwin;
            eout     <= #1 1'b0;
            end
  default : begin
            rsout    <= #1 1'b0;
            rwout    <= #1 1'b1;
            eout     <= #1 1'b0;
            end
  endcase 
endmodule
